<?php

namespace Drupal\commerce_nz\Plugin\Commerce\TaxNumberType;

use Drupal\commerce_tax\Plugin\Commerce\TaxNumberType\TaxNumberTypeBase;

/**
 * Provides the New Zealand GST number type.
 *
 * @CommerceTaxNumberType(
 *   id = "new_zealand_gst",
 *   label = "New Zealand GST",
 *   countries = {
 *     "NZ",
 *   },
 *   examples = {"096-259-824"}
 * )
 */
class NewZealandGst extends TaxNumberTypeBase {

  /**
   * {@inheritdoc}
   */
  public function validate($tax_number) {
    return $this->checkIrd((int) $this->canonicalize($tax_number));
  }

   /**
   * Check that NZ IRD # meets the IRD pattern spec.
   **
   * @param int $ird
   *   GST number to check.
   *
   * @return bool
   *   TRUE if the number meets the spec, FALSE if not.
   */
  private function checkIrd(int $ird): bool {
    // Should be 8 characters in length, it arrives as an int.
    if ($ird < 10000000 || $ird > 99999999) {
      return FALSE;
    }

    $weights = [2, 7, 6, 5, 4, 3, 2];
    $sird = (string) $ird;
    $sum = 0;

    for ($i = 0; $i < 7; ++$i) {
      $sum += $sird[$i] * $weights[$i];
    }
    $remainder = ($sum % 11);
    $check_digit = 11 - $remainder;

    if ($sird[7] == $check_digit) {
      return TRUE;
    }

    return FALSE;
  }

}
